import { nextTick, ref, useTemplateRef } from "vue";
import { getDataType } from "@/assets/js/utils.js";
import {
  getQueryCriteria,
  setQueryCriteria,
} from "@/assets/js/useQueryCriteria.js";

/**
 * @param api {Function} 接口函数
 * @param options {Object?: {callback, params, defaultSearchForm, immediate, usePagination, key}} 配置项
 * @param options.callback {Function?} 回调函数（返回值【第一个参数表格数据，第二个参数后台返回的所有数据】）
 * @param options.params {Object?} 其它接口参数
 * @param options.defaultSearchForm {Object?} searchForm默认值
 * @param options.immediate {Boolean?} 是否立即执行接口函数（默认是）
 * @param options.usePagination {Boolean?} 是否使用分页（默认是）
 * @param options.clearSelection {Boolean?} 调用resetPagination是是否清空表格选择数据（默认是）
 * @param options.key {String?} 返回的存放数组的key（默认list）
 * @param options.isStorageQueryCriteria {Boolean?} 是否保存查询条件（默认是）
 * @param options.tabsActiveName {String?} 存在tabs组件时，当前tabs的activeName，用于缓存查询条件
 * @return {Object} 返回对象包含以下属性：list 表格数据，pagination 分页数据，searchForm 搜索表单数据，tableRef 表格实例，getData 获取数据函数，resetPagination 重置分页函数
 */

export default function useListData(api, options = {}) {
  if (getDataType(api) !== "Function") throw new Error("api必须是一个函数");
  if (getDataType(options) !== "Object")
    throw new Error("options必须是一个对象");
  if (options.immediate && getDataType(options.immediate) !== "Boolean")
    throw new Error("options.immediate必须是一个布尔值");
  if (options.usePagination && getDataType(options.usePagination) !== "Boolean")
    throw new Error("options.usePagination必须是一个布尔值");
  if (options.key && getDataType(options.key) !== "String")
    throw new Error("options.key必须是一个字符串");
  if (
    options.callback &&
    getDataType(options.callback) !== "Function" &&
    getDataType(options.callback) !== "AsyncFunction"
  )
    throw new Error("options.callback必须是一个函数");
  if (
    options.defaultSearchForm &&
    getDataType(options.defaultSearchForm) !== "Object"
  )
    throw new Error("options.defaultSearchForm必须是一个对象");
  if (
    options.clearSelection &&
    getDataType(options.clearSelection) !== "Boolean"
  )
    throw new Error("options.clearSelection必须是一个布尔值");
  if (options.params && getDataType(options.params) !== "Object")
    throw new Error("options.otherParams必须是一个对象");
  if (
    options.isStorageQueryCriteria &&
    getDataType(options.isStorageQueryCriteria) !== "Boolean"
  )
    throw new Error("options.isStorageQueryCriteria必须是一个布尔值");
  if (
    options.tabsActiveName &&
    getDataType(options.tabsActiveName) !== "String"
  )
    throw new Error("options.tabsActiveName必须是一个字符串");
  const immediate = options.immediate ?? true;
  const usePagination = options.usePagination ?? true;
  const key = options.key ?? "list";
  const defaultSearchForm = options.defaultSearchForm ?? {};
  const clearSelection = options.clearSelection ?? true;
  const isStorageQueryCriteria = options.isStorageQueryCriteria ?? true;
  const list = ref([]);
  const queryCriteria = getQueryCriteria();
  const pagination = ref(
    queryCriteria.pagination || {
      currentPage: 1,
      pageSize: 10,
      total: 0,
    }
  );
  const searchForm = ref(queryCriteria.searchForm || defaultSearchForm);
  const tableRef = useTemplateRef("tableRef");
  const getData = async (params = {}) => {
    const resData = await api({
      ...(usePagination
        ? {
            curPage: pagination.value.currentPage,
            limit: pagination.value.pageSize,
          }
        : {}),
      ...searchForm.value,
      ...(options.params || {}),
      ...(getDataType(params) === "Object" ? params : {}),
    });
    list.value = usePagination ? resData.page[key] : resData[key];
    if (usePagination) pagination.value.total = resData.page.totalCount;
    options.callback && options.callback(list.value, resData);
    !usePagination &&
      clearSelection &&
      tableRef.value &&
      tableRef.value.clearSelection();
    if (isStorageQueryCriteria) {
      setQueryCriteria({
        searchForm: searchForm.value,
        pagination: pagination.value,
        tabsActiveName: options.tabsActiveName,
      });
    }
  };
  immediate && getData().then();
  const resetPagination = async (params) => {
    list.value = [];
    pagination.value = {
      currentPage: 1,
      pageSize: 10,
      total: 0,
    };
    await nextTick();
    await getData(params);
    clearSelection && tableRef.value && tableRef.value.clearSelection();
  };
  return {
    list,
    pagination,
    searchForm,
    tableRef,
    getData: async (params) => await getData(params),
    resetPagination: async (params) => await resetPagination(params),
  };
}
